/**************************************************************************
        ------------------------------------------------------------
                           jconv   Version 1.0
        ------------------------------------------------------------

                       copyright Andreas Mock 1999

                      Andreas Mock <AMock@osram.com>

        ------------------------------------------------------------

                   based on the original source code and
                       with friendly permission of

                     J. J. Lehett <JLehett@land-j.com>

        ------------------------------------------------------------


        You are permitted to use the jconv package in the
        terms of the GNU General Public License.


                    GNU GENERAL PUBLIC LICENSE
                    Version 2, June 1991


        See the COPYRIGHT file in this package.

**************************************************************************/

#include "jconv.h"
#include <stdio.h>            
#include <stdlib.h>
#include <string.h> 
#include <math.h>
#include <unistd.h>

#define MAXERR 255
#define MB_OK 0

char		popupListStringForFunction[10000];  
tRecEntry	recEntrys[MAX_RECORDS];
int		dataLengths[MAX_RECORDS]; 
short		dataLengthsFields[MAX_FIELDS];
short		curFieldTypes[MAX_FIELDS];
int		BinaryDataSize;
char		BinaryDataString[4000];
char		tempFieldNames[MAX_FIELDS][55];


int main(int argc, char* argv[])
{
	char		infofile[FILENAME_MAX];
	char		outfile[FILENAME_MAX];
	char		infile[FILENAME_MAX];
	char		title[32];
	char		o = ' ';
	char		encode = ' ';
	extern char	*optarg;
	extern int optind, opterr, optopt;

	opterr = 0;		/* no option processing error */

	strcpy(infofile, "");
	strcpy(outfile, "");
	strcpy(infile, "");

	delimChar = ',';	/* standard */
	m_CommaDelim = 1;	/* standard */
	m_UseInfoFile = 0;

	while((o = getopt(argc, argv, "edvht:i:s")) != EOF) {
		switch(o) {
		case 'e':	encode = 'e';
				break;
		case 'd':	encode = 'd';
				break;
		case 's':	m_CommaDelim = 0;
				break;
		case 't':	strncpy(title, optarg, 32);
				title[31] = 0;
				strcpy(m_DBName, title);
				break;
		case 'i':	strcpy(infofile, optarg);
				strcpy(m_InfoPath, optarg);
				m_UseInfoFile = 1;
				break;
		case 'h':	showsynopsis();
				exit(0);
		case 'v':	showversion();
				exit(0);
		case '?':	fprintf(stderr, "Error while option processing\n");
				showsynopsis();
				exit(1);
		}
	}
	

	if (optind != argc - 2) {
		fprintf(stderr, "Error: You must give a filename\n");
		showsynopsis();
		exit(1);
	}

	strcpy(infile, argv[optind]);
	strcpy(outfile, argv[optind + 1]);

	if(strlen(m_DBName) == 0) {
		strncpy(m_DBName, basename(outfile), 32);
		m_DBName[31] = 0;
	}

	switch(encode) {
	case ' ':	fprintf(stderr, "Error: You should specify what you want to do\n");
			showsynopsis();
			exit(1);
	case 'e':	strcpy(m_CSVPath, infile);
			strcpy(m_PDBPath, outfile);
			return(CSVToJFile());
	case 'd':	strcpy(m_PDBPath, infile);
			strcpy(m_CSVPath, outfile);
			return(JFileToCSV());

	}

	return(0);
}



void DecodeField(unsigned char* inData, char* outData, int type, int len)
{
	int loc, i;
	long	intHold;
	unsigned char   holdChr[2];

	switch(type)
	{
		case FLDTYPE_BINARY:
			holdChr[1] = '\0';
			for(i=0; i<len; i++)
			{
				holdChr[0] = (unsigned char)(inData[i]);
				sprintf(&(outData[i*2]), "%02X", holdChr[0]);
			}
			break;
		
		default:
			loc = 0;
			while((inData[loc++] != '\0') && (loc < len))
			{                        
				if(inData[loc-1] == '"')
				{
				 	inData[loc-1] = '\'';
				}     
				if((inData[loc-1] == '\n') || (inData[loc-1] == '\r') )
				{
			 		inData[loc-1] = ' ';
				}   
			}
			for(i=0; i<=len;i++)
			{
				outData[i] = inData[i];
			}
			outData[len] = '\0';

			break;
	}
}

int JFileToCSV()
{
	char		errorStr[MAXERR];            
	int		i,j;
	FILE*		input;
	FILE*		output;
	FILE*		info;
	tHeader		header;
	WORD		oddity = 0x008C;  
	int 		numNonDeletedRecs;  	
	JFileAppInfoType	appInfo;
	unsigned char 	dataDump[MAX_RECORD_LENGTH]; 
	long 		dumpLoc;  
	int		curDumpLoc;
	char*	 	tempPopupPtr; 
	int		usesInfoFile;
	int		fldLens[MAX_FIELDS];
	unsigned char	holdChr[2];
	char		endMsg[2048];
	char		recNumStr[10];
	int		err;
	int		k;
	int		ij;

	if(m_CommandLineComma == 0)
		delimChar = ';';

	usesInfoFile = m_UseInfoFile;

	if((input = fopen(m_PDBPath, "rb")) == NULL)
	{
		
		MessageBox2( "Error - Can't open pdb file", NULL, MB_OK );
		return(1);
	}
	if((output = fopen(m_CSVPath, "w")) == NULL)
	{
		
		MessageBox2( "Error - Can't open csv file", NULL, MB_OK );
		return(1);
	}
	if(usesInfoFile)
	{
		if((info = fopen(m_InfoPath, "w")) == NULL)
		{
			MessageBox2( "Error - Can't open ifo file", NULL, MB_OK );
			return(1);
		}
	}



 	fread(&header, 1, 78, input);	
	header.nRecs = FlipWord(header.nRecs);  
	numNonDeletedRecs = header.nRecs;


	for(i=0; i<header.nRecs; i++)
	{
		err = fread(&recEntrys[i].rp.ofs, 1, sizeof(DWORD), input);
		err = fread(&recEntrys[i]._id, 1, sizeof(DWORD), input);   

		if(recEntrys[i]._id & 0x00000080)
		{
		 	numNonDeletedRecs--;
		}
	}   
	
	fread(&oddity, 1, sizeof(oddity), input);



	if(	ReadInJFileAppInfo(&appInfo, input, info) == 0)
	{
		return(1);
	}


	appInfo.numFields = FlipWord(appInfo.numFields); 

	for(i=0; i<appInfo.numFields; i++)
	{
	     appInfo.fieldTypes[i] = FlipWord(appInfo.fieldTypes[i]);   
	     appInfo.showDBColumnWidths[i] = FlipWord(appInfo.showDBColumnWidths[i]);  
		 appInfo.fieldExtraData[i] = SwapDWordToIntel(appInfo.fieldExtraData[i]);
		 appInfo.fieldExtraData2[i] = SwapDWordToIntel(appInfo.fieldExtraData2[i]);
	} 
	
	appInfo.firstColumnToShow = FlipWord(appInfo.firstColumnToShow); 
	appInfo.showDataWidth = FlipWord(appInfo.showDataWidth);	
	appInfo.flags = FlipWord(appInfo.flags);	

		


	if(usesInfoFile)
	{
		
		// first line is the field types	
		for(i=0; i<appInfo.numFields; i++)
		{
			fprintf(info, "%i ", appInfo.fieldTypes[i]);
	 	}                                               
	 	fprintf(info, "\n");    
	 	
		// second line is the field widths	
		for(i=0; i<appInfo.numFields; i++)
	 	{
	 		fprintf(info, "%i ", appInfo.showDBColumnWidths[i]);
	 	}                                               
		fprintf(info, "\n");   
 	
		// third line is the data entry screen field width
	 	fprintf(info, "%i\n", appInfo.showDataWidth);  
	 	
	 	// 4th line is which column to display as first movable column in the database screen
	 	fprintf(info, "%i\n", appInfo.firstColumnToShow); 

	 	// 5th line is appInfo flags variable
	 	fprintf(info, "%i\n", appInfo.flags); 
	 
		// 6th and 7th lines are the field extra datas
		for(i=0; i<appInfo.numFields; i++)
	 	{
	 		fprintf(info, "%i ", appInfo.fieldExtraData[i]);
	 	}                                               
		fprintf(info, "\n"); 
		for(i=0; i<appInfo.numFields; i++)
	 	{
	 		fprintf(info, "%i ", appInfo.fieldExtraData2[i]);
	 	}                                               
		fprintf(info, "\n"); 

		// and before the popup's write out the binary data string
		for(i=0; i<BinaryDataSize; i++)
		{
			holdChr[0] = (unsigned char)(BinaryDataString[i]);
			fprintf(info, "%02X", holdChr[0]);
		}
		fprintf(info, "\n"); 

	 	tempPopupPtr = appInfo.popupLists;
		
		while(strlen(tempPopupPtr) != 0)
		{
			fprintf(info, "%s\n", tempPopupPtr);
				tempPopupPtr += (strlen(tempPopupPtr)+1);	
 		}
		

		fclose(info);
	}
	
    if(strlen(appInfo.fieldNames[0]) > 0)  
		fputs("\"", output);  
     
    for(i=0; i<strlen( appInfo.fieldNames[0]); i++)
    {             
    	if(appInfo.fieldNames[0][i] == '"')
    		fputc('"', output);	
    	fputc(appInfo.fieldNames[0][i], output);
    }

    if(strlen(appInfo.fieldNames[0]) > 0)  
		fputs("\"", output);     

	for(i=1; i<appInfo.numFields; i++)
	{
		fputc(delimChar, output);    
		if(strlen(appInfo.fieldNames[i]) > 0)  
			fputs("\"", output);     
	
	   	for(j=0; j<strlen( appInfo.fieldNames[i]); j++)
   	 	{               
   	    	if(appInfo.fieldNames[i][j] == '"')
    			fputc('"', output);	
    		fputc(appInfo.fieldNames[i][j], output);
    	}
   
		if(strlen(appInfo.fieldNames[i]) > 0)  
			fputs("\"", output);    
	}
	fputs("\n", output);

	dumpLoc = 0;


	//!!!!! 10 extra bytes, but why????
	//for(j=0; j<10; j++)
	//	fgetc(input);



	for(i=0; i<numNonDeletedRecs; i++)
	{   
		//fputs("new rec read\n", output);

		for(j=0; j<appInfo.numFields*2; j++)
		{
			dataDump[j] = fgetc(input);
		}

		// read in the field lengths
		for(j=0; j<appInfo.numFields; j++)
		{
			fldLens[j] = (int)((dataDump[j*2] - '\0')*256) +  (int)((dataDump[j*2+1] - '\0'));
			//fprintf(output, "length of field %i is %i\n", j, fldLens[j]);
		}


		dumpLoc = appInfo.numFields * 2;
		curDumpLoc = dumpLoc;


		for(j=0; j<appInfo.numFields; j++)
		{
			for(k = 0; k < fldLens[j]; k++)
			{
				dataDump[curDumpLoc++] = fgetc(input);
				//fputc(dataDump[curDumpLoc-1], output);
			}
		}

		for(j=0; j<appInfo.numFields; j++)
		{
			//fprintf(output, "found type %i, and string = %s and len=%i\n", appInfo.fieldTypes[j],&dataDump[dumpLoc], fldLens[j]);
			DecodeField(&dataDump[dumpLoc], tempRec.fieldData[j], appInfo.fieldTypes[j], fldLens[j]);
			dumpLoc += fldLens[j];
		}

		for(ij = 0; ij<appInfo.numFields; ij++)
		{
			if(ij > 0)
				fputc(delimChar, output); 
			fputs("\"", output);
			fputs(tempRec.fieldData[ij], output);
			fputs("\"", output);
		}

		fputs("\n", output);
	}

	// ok everything should be read in close the input file
	fclose(input);                                                                
	fclose(output); 



	sprintf(recNumStr, "%d", numNonDeletedRecs);
	strcpy(endMsg, "Conversion of ");
	strcat(endMsg, recNumStr);
	strcat(endMsg, " records in ");
	strcat(endMsg, m_PDBPath);
	strcat(endMsg, " to ");
	strcat(endMsg, m_CSVPath);
	strcat(endMsg, " was successful!");
	MessageBox( endMsg, NULL, MB_OK );

}

int CSVToJFile()
{
	char		errorStr[MAXERR];
	char		errorNum[50];
	unsigned long	currentOfs;   
	int		i,j;
	char		dbName[200];
	int		usesInfoFile;
	tHeader		header;
	char 		inputString[MAX_RECORD_LENGTH];  
	int 		inputStringLoc, inputStringLen;  
	JFileAppInfoType	appInfo;
	int		fieldNameLocs[MAX_FIELDS];  
	int		isInQuote = 0;                   
	int		wasQuoted = 0; 
	char 		tempLine[MAX_DATA_LENGTH+1];       
	char 		tempLine2[MAX_DATA_LENGTH+1];   	
	int		currentRecord;  
	int 		currentField; 
	int		dataLocs[MAX_FIELDS];  
	JBaseRecordType		record;  
	char*		returnPtr = NULL;
	int		u = 0;

   
	WORD		oddity = 0x008C;  // unknown byte in the pdb struct


	char	endMsg[2048];
	char	recNumStr[10];

	FILE*	input;
	FILE*	output;
	FILE*	info;

	char* nullPtr;

	if(m_CommandLineComma == 0)
		delimChar = ';';

	nullPtr = NULL;

	usesInfoFile = m_UseInfoFile;



	if((input = fopen(m_CSVPath, "r")) == NULL)
	{
		// handle the error here
		MessageBox2( "Error - Can't open csv file", NULL, MB_OK );
		return;
	}
	if((output = fopen(m_PDBPath, "wb")) == NULL)
	{
		//handle the error here
		MessageBox2( "Error - Can't open pdb file", NULL, MB_OK );
		return;
	}
	if(usesInfoFile)
	{
		if((info = fopen(m_InfoPath, "r")) == NULL)
		{
			//handle the error here
			//MessageBox2( "Error - Can't open ifo file", NULL, MB_OK );
			usesInfoFile = 0;
		}
	}


	// need to find DB name somehow
	strcpy(dbName, m_PDBPath);
	dbName[30]= '\0';

	strcpy(header.sTitle, dbName);    
	
	// get the field names

	fgets(inputString, MAX_RECORD_LENGTH, input);
	if(strlen(inputString) > 3999)
	{
		strcpy(errorStr, "Field Names record too long");
		MessageBox2( errorStr, NULL, MB_OK );
		fclose(input);
		fclose(output);
		if(usesInfoFile)
			fclose(info);
		return;
	}    
	

	inputStringLoc = 0;                 
	appInfo.numFields = 0;
	fieldNameLocs[0] = 0;      
	inputStringLen = strlen(inputString);   
	
	while(inputStringLoc < inputStringLen)
	{                
		if((inputString[inputStringLoc] == '"') && (inputString[inputStringLoc+1] == '"') && (isInQuote == 1))
		{                                     
			inputStringLoc += 2;
			continue;
		}     
	
	    if((inputString[inputStringLoc] == '"'))
		{                                     
 			if(isInQuote == 1)
 			{                 
 				isInQuote = 0;
 			}
 			else
 			{  
 				wasQuoted = 1;
 			 	isInQuote = 1;
 			}
		}  

		if((inputString[inputStringLoc] == delimChar) && (isInQuote == 0))
		{                        
			fieldNameLocs[appInfo.numFields+1] = inputStringLoc+1;
			if(fieldNameLocs[appInfo.numFields+1] - fieldNameLocs[appInfo.numFields] > MAX_FIELD_NAME_LENGTH+3)
			{	
				sprintf(errorNum, "%d", appInfo.numFields + 1);
				strcpy(errorStr, "Field Name: ");
				strcat(errorStr, errorNum);
				strcat(errorStr, " too long");
				MessageBox2( errorStr, NULL, MB_OK );
				fclose(input);
				fclose(output);
				if(usesInfoFile)
					fclose(info);
				return;
			}
			appInfo.numFields++;
			inputString[inputStringLoc] = '\0';
		}         
		if(inputString[inputStringLoc] == '\n')
		{               
			inputString[inputStringLoc] = '\0';
			fieldNameLocs[appInfo.numFields+1] = inputStringLoc+1;
			if(fieldNameLocs[appInfo.numFields+1] - fieldNameLocs[appInfo.numFields] > MAX_FIELD_NAME_LENGTH+3)
			{	
				sprintf(errorNum, "%d", appInfo.numFields + 1);
				sprintf(errorStr, "Field Name: %s too long",
					errorNum);
				MessageBox2( errorStr, NULL, MB_OK );
				fclose(input);
				fclose(output);
				if(usesInfoFile)
					fclose(info);
				return;
			}
		}
		inputStringLoc++;	
	}  
	

	appInfo.numFields++;

	for(i=0; i<MAX_FIELDS; i++)
	{
		curFieldTypes[i] = 1;
	}

	if(usesInfoFile)
	{
		ReadJFileInfoFileFieldTypesOnly(&appInfo, info);
		fclose(info);
		if((info = fopen(m_InfoPath, "r")) == NULL)
		{
			//handle the error here
			usesInfoFile = 0;
		}
	}


	if(appInfo.numFields > MAX_FIELDS)
	{
		sprintf(errorNum, "%d", appInfo.numFields);
		sprintf(errorStr, "ERROR: This csv file contains %s  fields.  50 fields is the maximum limit for JFile.", errorNum);
		MessageBox2( errorStr, NULL, MB_OK );
		fclose(input);
		fclose(output);
		if(usesInfoFile)
			fclose(info);
		return;
	}

	for(i=0; i<appInfo.numFields; i++)
	{                                                         
	 	strcpy(tempFieldNames[i], &inputString[fieldNameLocs[i]]);

		
	}              
	
	// now clear out any " occurence problems
	for(i=0; i<appInfo.numFields; i++)
	{
	  	strcpy(tempLine, tempFieldNames[i]);
	  	
	  	//see if we're quoted
	  	if((tempLine[0] == '"'))
	  	{
	  	 	// we have a quoted line....
	  	 	strcpy(tempLine2, &tempLine[1]);  
	  	 	tempLine2[strlen(tempLine2) - 1] = '\0';  
	  	 	strcpy(tempLine, tempLine2);
	  	}                               
	  	
	  	// ok now take out any double quotes
	  	for(j=0; j<strlen(tempLine); j++)
	  	{
	  		if((tempLine[j] == '"') && (tempLine[j+1] == '"'))
	 	 	{                        
	 	 		tempLine[j+1] = '\0';
	 	 		strcpy(tempLine2, "");
	  		 	// we have a double quote
	   	        strcat(tempLine2, tempLine);
	   	        strcat(tempLine2, &tempLine[j+2]);
	   	        strcpy(tempLine, tempLine2);
	   	      
	  		}  	  		
	  	}    
	  	strcpy(appInfo.fieldNames[i], tempLine);
	
		//MessageBox( tempFieldNames[i], NULL, MB_OK );
		
		if(strlen(appInfo.fieldNames[i]) > MAX_FIELD_NAME_LENGTH)
		{	
			sprintf(errorStr, "Field Name:  too long");
			MessageBox2( errorStr, NULL, MB_OK );
			fclose(input);
			fclose(output);
			if(usesInfoFile)
				fclose(info);
			return;
		}
		
	}


	currentRecord = -1;                 



	returnPtr = fgets(inputString, MAX_RECORD_LENGTH, input);

	while(	returnPtr != NULL)
	{   

		//printf("%i\n", currentRecord+1);      
		currentRecord++;
		currentField = 0; 
		inputStringLoc = 0;   

		inputStringLen = strlen(inputString);    
		if(inputString[inputStringLen-1] != '\n')
		{
			inputString[inputStringLen] = '\n';
			inputString[inputStringLen+1] = '\0';
			inputStringLen++;
		}
		//inputString[inputStringLen+1] = '\0';
		dataLocs[0] = 0;
		while(inputStringLoc < inputStringLen)
		{                  
			if((inputString[inputStringLoc] == '"') && (inputString[inputStringLoc+1] == '"') && (isInQuote == 1))
			{                                     
				inputStringLoc += 2;
				continue;
			}  	   
		
		    if((inputString[inputStringLoc] == '"'))
			{                                     
 				if(isInQuote == 1)
 				{                 
 					isInQuote = 0;
 				}
 				else
 				{  
 					wasQuoted = 1;
 				 	isInQuote = 1;
 				}
			}


			if((inputString[inputStringLoc] == delimChar)  && (isInQuote == 0))
			{                                     
				dataLocs[currentField+1] = inputStringLoc+1;
				if(dataLocs[currentField+1] - dataLocs[currentField] > MAX_DATA_LENGTH)
				{
					sprintf(errorNum, "%d", currentRecord+1);
					sprintf(errorStr, "Field data too long at record: %s", errorNum);
					MessageBox2( errorStr, NULL, MB_OK );
					fclose(input);
					fclose(output);
					if(usesInfoFile)
						fclose(info);
					return;
				}
				currentField++;
				inputString[inputStringLoc] = '\0';
			}         
			if(inputString[inputStringLoc] == '\n')
			{  
				if(isInQuote)
				{
					sprintf(errorNum, "%d", currentRecord+1);
					sprintf(errorStr, "Unclosed quoted string at record: %s", errorNum);
					MessageBox2( errorStr, NULL, MB_OK );
					fclose(input);
					fclose(output);
					if(usesInfoFile)
						fclose(info);
					return;
				}
				if(inputStringLoc - dataLocs[currentField] > MAX_DATA_LENGTH)
				{
					sprintf(errorNum, "%d", currentRecord+1);
					sprintf(errorStr, "Field data too long
 at record: %s", errorNum);
					MessageBox2( errorStr, NULL, MB_OK );
					fclose(input);
					fclose(output);
					if(usesInfoFile)
						fclose(info);
					return;
				}				  
				inputString[inputStringLoc] = '\0';

				while(currentField < appInfo.numFields)
				{
					dataLocs[currentField+1] = inputStringLoc+1;
					currentField++;
					inputString[inputStringLoc++] = '\0';
				}
			}
			inputStringLoc++;	
		}   
		
          
		           
		if(currentField != appInfo.numFields)
		{
			sprintf(errorNum, "%d", currentRecord+2);
			sprintf(errorStr, "Invalid number of fields (commas) at CSV line #: %s", errorNum);
			MessageBox2( errorStr, NULL, MB_OK );
			fclose(input);
			fclose(output);
			//fclose(info);
			return;
		}            
		
		for(i=0; i<appInfo.numFields; i++)
		{                                                         
		 	strcpy(record.fieldData[i], &inputString[dataLocs[i]]);
		}           
		
		// now clear out any " occurence problems
		for(i=0; i<appInfo.numFields; i++)
		{
		  	strcpy(tempLine, record.fieldData[i]);
		  	
		  	//see if we're quoted
		  	if((tempLine[0] == '"'))
		  	{
		  	 	// we have a quoted line....
		  	 	strcpy(tempLine2, &tempLine[1]);  
		  	 	tempLine2[strlen(tempLine2) - 1] = '\0';  
		  	 	strcpy(tempLine, tempLine2);
		  	}                               
		  	
		  	// ok now take out any double quotes
		  	for(j=0; j<strlen(tempLine); j++)
		  	{
		  		if((tempLine[j] == '"') && (tempLine[j+1] == '"'))
		 	 	{                        
		 	 		tempLine[j+1] = '\0';
		 	 		strcpy(tempLine2, "");
		  		 	// we have a double quote
		   	        strcat(tempLine2, tempLine);
		   	        strcat(tempLine2, &tempLine[j+2]);
		   	        strcpy(tempLine, tempLine2);
		   	        
		  		}  	  		
		  	}       
		  	strcpy(record.fieldData[i], tempLine);
		}     
		
        //!! now convert from 'string' to any appropriate field type            
		
		// initial length from 2 bytes per field length in start of record
		dataLengths[currentRecord] = appInfo.numFields * 2;

		for(i=0; i<appInfo.numFields; i++)
		{
			switch(curFieldTypes[i])
			{
				case FLDTYPE_BINARY:
					//!! to be completed!
					dataLengths[currentRecord] += (strlen(record.fieldData[i]))/2;
					break;
			
				default:
					dataLengths[currentRecord] += (strlen(record.fieldData[i]) + 1);
					break;
			}
		}
		
		returnPtr = fgets(inputString, MAX_RECORD_LENGTH, input);
	} 



	for(i=0; i<MAX_FIELDS; i++)
	{
	 	appInfo.fieldTypes[i] = 1;                          
	 	appInfo.fieldTypes[i] = FlipWord(appInfo.fieldTypes[i]);
	 	appInfo.showDBColumnWidths[i] = 80;                             
	 	appInfo.showDBColumnWidths[i] = FlipWord(appInfo.showDBColumnWidths[i]);
	}
	appInfo.sort1Field = 0;
	appInfo.sort1Field = FlipWord(appInfo.sort1Field);
	appInfo.sort2Field = 0;
	appInfo.sort2Field = FlipWord(appInfo.sort2Field);
	appInfo.sort3Field = 0;                                 
	appInfo.sort3Field = FlipWord(appInfo.sort3Field);
	appInfo.findField = 0;                                  
	appInfo.findField = FlipWord(appInfo.findField);
	appInfo.filterField = 0;                                
	appInfo.filterField = FlipWord(appInfo.filterField);
	strcpy(appInfo.findString, "");
	strcpy(appInfo.filterString, "");
	appInfo.firstColumnToShow = 1;                          
	appInfo.firstColumnToShow = FlipWord(appInfo.firstColumnToShow);
	appInfo.showDataWidth = 80;
	appInfo.showDataWidth = FlipWord(appInfo.showDataWidth);
	appInfo.version = CURRENT_VERSION;
	appInfo.version = FlipWord(appInfo.version);
	appInfo.popupLists = NULL;

	if(usesInfoFile)   
	{
		// overwrite those info blocks that are in the info File
		if(ReadJFileInfoFile(&appInfo, info, popupListStringForFunction, output) == 0)
		{
			fclose(input);
			fclose(output);
			fclose(info);
			return;
		}
		appInfo.popupLists =  popupListStringForFunction;
	}      
	else
	{
		strcpy(BinaryDataString, "EndJFileData");
		BinaryDataSize = strlen(BinaryDataString)+1;
	}
	
	appInfo.numFields = FlipWord(appInfo.numFields);      
	
	header.nRecs = currentRecord+1;
	header.ofsAttributes = 78 + 2 + 8*header.nRecs;
    

    if(!usesInfoFile)
    {
		currentOfs = 78 + 8*header.nRecs + 2 + JFILEAPPINFOSIZE+ BinaryDataSize;  
	}
	else
	{
		currentOfs = 78 + 8*header.nRecs + 2 + FindJFileAppInfoSize(&appInfo) ;	
	}
	
	recEntrys[0].rp.ofs = currentOfs;
	for(i=1; i<=currentRecord; i++)
	{                      
		currentOfs += dataLengths[i-1];

	 	recEntrys[i].rp.ofs = currentOfs;
	 	recEntrys[i]._id = 0x000000;
	}                          
	
	// ok everything should be read in close the input file
	fclose(input);



	
	// and set up to do the output file
	header.nRecs = currentRecord+1;
	header.dwType = 0x4A664462;
	header.dwCreator = 0x4A46696c;
	header.wVersion = 1;            
	header.wAttr = 8;
	header.tCreated = 0x33332675;
	header.tModded = 0x33332675;

	// swap things
	header.tCreated = SwapDWordToMotor(header.tCreated);
	header.tModded = SwapDWordToMotor(header.tModded);
	header.wVersion = FlipWord(header.wVersion);
	header.ofsAttributes = SwapDWordToMotor(header.ofsAttributes);
	header.dwType = SwapDWordToMotor(header.dwType);
	header.dwCreator = SwapDWordToMotor(header.dwCreator);
	header.wAttr = FlipWord(header.wAttr);
	header.nRecs = FlipWord(header.nRecs);          
	

	for(i=0; i<=currentRecord; i++)
	{
		recEntrys[i].rp.ofs = SwapDWordToMotor(recEntrys[i].rp.ofs);

	}   

	if(strlen(m_DBName) > 0)
	{
		strcpy(header.sTitle, m_DBName);
	}

	fwrite(&header, 1, 78, output);	

	for(i=0; i<=currentRecord; i++)
	{
		fwrite(&recEntrys[i].rp.ofs, 1, sizeof(DWORD), output);
		fwrite(&recEntrys[i]._id, 1, sizeof(DWORD), output);
	}    
	
	fwrite(&oddity, 1, sizeof(oddity), output);	
	
	if(!usesInfoFile)
	{	
		fwrite(&appInfo, 1, JFILEAPPINFOSIZE, output);
		fwrite(BinaryDataString, 1, BinaryDataSize, output);
	}
    else
    {
		fwrite(&appInfo, 1, (JFILEAPPINFOSIZE)-4, output);
		fwrite(appInfo.popupLists, 1, FindJFileAppInfoSize(&appInfo) - ((JFILEAPPINFOSIZE)-4) - BinaryDataSize, output);   
		fwrite(BinaryDataString, 1, BinaryDataSize, output);
    }
    
	if((input = fopen(m_CSVPath, "r")) == NULL)
	{
				sprintf(errorStr, "Could not open input file.");
				MessageBox2( errorStr, NULL, MB_OK );
				fclose(input);
				fclose(output);
				fclose(info);
				return;
	}	
	
	header.tCreated = SwapDWordToMotor(header.tCreated);
	header.tModded = SwapDWordToMotor(header.tModded);
	header.wVersion = FlipWord(header.wVersion);
	header.ofsAttributes = SwapDWordToMotor(header.ofsAttributes);
	header.dwType = SwapDWordToMotor(header.dwType);
	header.dwCreator = SwapDWordToMotor(header.dwCreator);
	header.wAttr = FlipWord(header.wAttr);
	header.nRecs = FlipWord(header.nRecs);
	appInfo.numFields = FlipWord(appInfo.numFields);

	// get the field names
	fgets(inputString, 4000, input);

	currentRecord = -1;                 
	while(	fgets(inputString, 4000, input) != NULL)
	{             

		currentRecord++;
		currentField = 0; 
		inputStringLoc = 0;    
		inputStringLen = strlen(inputString);   
		if(inputString[inputStringLen-1] != '\n')
		{
			inputString[inputStringLen] = '\n';
			inputString[inputStringLen+1] = '\0';
			inputStringLen++;
		}
		//inputString[inputStringLen+1] = '\0';
		//dataLengths[currentRecord] = inputStringLen;
		dataLocs[0] = 0;
		while(inputStringLoc < inputStringLen)
		{                  
		
			if((inputString[inputStringLoc] == '"') && (inputString[inputStringLoc+1] == '"') && (isInQuote == 1))
			{                                     
				inputStringLoc += 2;
				continue;
			}     
		    if((inputString[inputStringLoc] == '"'))
			{                                     
 				if(isInQuote == 1)
 				{                 
 				isInQuote = 0;
 				}
 				else
 				{  
 					wasQuoted = 1;
 				 	isInQuote = 1;
 				}
 			}                
 			
			if((inputString[inputStringLoc] == delimChar)  && (isInQuote == 0))
			{                                     
				dataLocs[currentField+1] = inputStringLoc+1;
				if(dataLocs[currentField+1] - dataLocs[currentField] > MAX_DATA_LENGTH)
				{
					sprintf(errorNum, "%d", currentRecord);
					sprintf(errorStr, "Field data at record: %s  too long", errorNum);
					MessageBox2( errorStr, NULL, MB_OK );
					fclose(input);
					fclose(output);
					fclose(info);
					return;
				}
				currentField++;
				inputString[inputStringLoc] = '\0';
			}         
			if(inputString[inputStringLoc] == '\n')
			{                                     
				inputString[inputStringLoc] = '\0';

				while(currentField < appInfo.numFields)
				{
					dataLocs[currentField+1] = inputStringLoc+1;
					currentField++;
					inputString[inputStringLoc++] = '\0';
				}
			}
			inputStringLoc++;	
		}   
		
		
		for(i=0; i<appInfo.numFields; i++)
		{                                                         
		 	strcpy(record.fieldData[i], &inputString[dataLocs[i]]);
		}     
		
		// now clear out any " occurence problems
		for(i=0; i<appInfo.numFields; i++)
		{
		  	strcpy(tempLine, record.fieldData[i]);
		  	
		  	//see if we're quoted
		  	if((tempLine[0] == '"'))
		  	{
		  	 	// we have a quoted line....
		  	 	strcpy(tempLine2, &tempLine[1]);  
		  	 	tempLine2[strlen(tempLine2) - 1] = '\0';  
		  	 	strcpy(tempLine, tempLine2);
		  	}                               
		  	
		  	// ok now take out any double quotes
		  	for(j=0; j<strlen(tempLine); j++)
		  	{
		  		if((tempLine[j] == '"') && (tempLine[j+1] == '"'))
		 	 	{                        
		 	 		tempLine[j+1] = '\0';
		 	 		strcpy(tempLine2, "");
		  		 	// we have a double quote
		   	        strcat(tempLine2, tempLine);
		   	        strcat(tempLine2, &tempLine[j+2]);
		   	        strcpy(tempLine, tempLine2);
		   	        
		  		}  	  		
		  	}       
		  	strcpy(record.fieldData[i], tempLine);
		} 
		
		for(j=0; j<appInfo.numFields; j++)
		{

			switch(curFieldTypes[j])
			{
				case FLDTYPE_BINARY:
					//!! to be completed!
					dataLengths[j] += (strlen(record.fieldData[j]))/2;
					dataLengthsFields[j] = (strlen(record.fieldData[j]))/2;


					for(u=0; u<(dataLengthsFields[j]*2); u=u+2)
					{
						if(	(record.fieldData[j][u] >= '0') && (record.fieldData[j][u] <= '9'))
						{
							record.fieldData[j][u/2] = 16*(record.fieldData[j][u] - '0');
						}
						else
						{
							record.fieldData[j][u/2] = 16*(record.fieldData[j][u] - 'A' + 10);
						}
						
						if(	(record.fieldData[j][u+1] >= '0') && (record.fieldData[j][u+1] <= '9'))
						{
							record.fieldData[j][u/2] += (record.fieldData[j][u+1] - '0');
						}
						else
						{
							record.fieldData[j][u/2] += (record.fieldData[j][u+1] - 'A' + 10);
						}
					}
					break;

				default:
					dataLengths[j] += (strlen(record.fieldData[j]) + 1);
					dataLengthsFields[j] = (strlen(record.fieldData[j]) + 1);
					break;
			}


		}

		for(j=0; j<appInfo.numFields; j++)
	 	{
			dataLengthsFields[j] = FlipWord(dataLengthsFields[j]);
	 	 	fwrite(&(dataLengthsFields[j]), 2, 1, output);
			dataLengthsFields[j] = FlipWord(dataLengthsFields[j]);
	 	}

		for(j=0; j<appInfo.numFields; j++)
	 	{
	 	 	fwrite(&record.fieldData[j], 1, dataLengthsFields[j], output);
	 	}
	}  
	
	fclose(input);                                                    
	fclose(output);	  
    
    if(usesInfoFile)
    {
			fclose(info);	
	}


	sprintf(recNumStr, "%d", currentRecord+1);
	sprintf(endMsg, "Conversion of %s  records in %s to %s was successful!",
			recNumStr, m_CSVPath, m_PDBPath);

	if(!m_IsCommandLine)
		MessageBox( endMsg, NULL, MB_OK );

}

int TypeIsString(int t)
{
	int tempT;

	tempT = t;

	if((tempT == FLDTYPE_STRING) || (tempT == FLDTYPE_LIST))
		return(1);
   
	return(0);
}

int ReadInJFileAppInfo(JFileAppInfoType * appInfo, FILE * input, FILE* info)
{
	int 				popupListLocation = 0;
	int				err;
	short				len;
	char				inData[4002];
	int				i=0;

	// read in all but the popup list
	// !! why is this 6 instead of 4????
	err = fread(appInfo, 1, (JFILEAPPINFOSIZE) - 4, input);

	// minimum of 4 bytes for the popup list string
	fread((&PopupListString[popupListLocation++]), 1, 1, input);
	fread((&PopupListString[popupListLocation++]), 1, 1, input);
	fread((&PopupListString[popupListLocation++]), 1, 1, input);
	fread((&PopupListString[popupListLocation++]), 1, 1, input);
	
	// now read in the popup list - it will end with 2 null's    
	if( (PopupListString[popupListLocation-3] != '\0') && (PopupListString[popupListLocation-4] != '\0'))
	{
		while( (PopupListString[popupListLocation-1] != '\0') || (PopupListString[popupListLocation-2] != '\0'))
		{
			fread((&PopupListString[popupListLocation++]), 1, 1, input);
		}
	}
	
	BinaryDataSize = 0;

	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
	fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);

	// now read in the 'binary' data ending in 'EndJFileData'
	while( (BinaryDataString[BinaryDataSize-13] != 'E') || (BinaryDataString[BinaryDataSize-12] != 'n')|| (BinaryDataString[BinaryDataSize-11] != 'd')|| (BinaryDataString[BinaryDataSize-10] != 'J')|| (BinaryDataString[BinaryDataSize-9] != 'F')|| (BinaryDataString[BinaryDataSize-8] != 'i')|| (BinaryDataString[BinaryDataSize-7] != 'l')|| (BinaryDataString[BinaryDataSize-6] != 'e')|| (BinaryDataString[BinaryDataSize-5] != 'D')|| (BinaryDataString[BinaryDataSize-4] != 'a')|| (BinaryDataString[BinaryDataSize-3] != 't')|| (BinaryDataString[BinaryDataSize-2] != 'a')|| (BinaryDataString[BinaryDataSize-1] != '\0'))
	{
		i++;
		fread((&BinaryDataString[BinaryDataSize++]), 1, 1, input);
		if(i>4000)
		{
			MessageBox2( "Error - Invalid AppInfo in .pdb file", NULL, MB_OK );
			return(1);
		}
		//popupListLocation++;
	}
	
	appInfo->popupLists = PopupListString;

	return(1);
	
}

int FindJFileAppInfoSize(JFileAppInfoType * appInfo)
{
 	int		appInfoSize = 0; 
 	char*		tempPopupPtr;
 	
 	appInfoSize = (JFILEAPPINFOSIZE - 4);        
 	
 	tempPopupPtr = appInfo->popupLists;
 
	if(strlen(tempPopupPtr) == 0)
	{
		appInfoSize+=4;	// add 4 for the 4 nulls needed for no popup list
	}
	else
	{
	 	while(strlen(tempPopupPtr) != 0)
		{
			appInfoSize += (strlen(tempPopupPtr)+1);
	 		tempPopupPtr += (strlen(tempPopupPtr)+1);	
	 	} 
     	appInfoSize++;    // extra null since popup lists end in 2 nulls
	}



	appInfoSize += BinaryDataSize;

	if(appInfoSize < JFILEAPPINFOSIZE)
	{
		appInfoSize = JFILEAPPINFOSIZE;
	}

 	
 	return(appInfoSize);
}

int ReadJFileInfoFileFieldTypesOnly(JFileAppInfoType * appInfo, FILE * info)
{

	char* 	tempPopupPtr; 
	int		i;
	int		temp;
	int 	numItems = 0;
	int		listLength = 0;
	char	errorStr[2048];

	// first line is the field types	
	for(i=0; i<appInfo->numFields; i++)
 	{
 		if(fscanf(info, "%i", &(temp)) != 1)
		{
			char errorStr[2048];
			strcpy(errorStr, "Invalid number of field types in info file.");
			MessageBox( errorStr, NULL, MB_OK );
			return(1);
		}
		curFieldTypes[i] = temp;
 		//curFieldTypes[i] = FlipWord(appInfo->fieldTypes[i]);
 	}                                               
 
	return(1);

}

int ReadJFileInfoFile(JFileAppInfoType * appInfo, FILE * info, char * popupListString, FILE* output)
{

	char* 	tempPopupPtr; 
	int		i;
	int		temp;
	int 	numItems = 0;
	int		listLength = 0;
	char	errorStr[2048];
	char	binStr[4000];
	int	binLen;

	// first line is the field types	
	for(i=0; i<appInfo->numFields; i++)
 	{
		if(fscanf(info, "%i", &(temp)) != 1)
		{
			char errorStr[2048];
			strcpy(errorStr, "Invalid number of field types in info file.");
			MessageBox( errorStr, NULL, MB_OK );
			return(1);
		}
 		//printf("%i " , appInfo->numFields);
		appInfo->fieldTypes[i] = temp;
		//if( (temp != FLDTYPE_STRING) && (temp != FLDTYPE_BOOLEAN) && (temp != FLDTYPE_DATE) &&
		//	(temp != FLDTYPE_INT) && (temp != FLDTYPE_FLOAT) && (temp != FLDTYPE_LIST) )
		//{
		//	printf("Error: Invalid field type in info file.\n");
		//	exit(0);
		//}

 		appInfo->fieldTypes[i] = FlipWord(appInfo->fieldTypes[i]);
		//printf("%i " , appInfo->numFields);
 	}                                               
 	//printf("%i " , appInfo->numFields);
	// second line is the field widths	
	for(i=0; i<appInfo->numFields; i++)
 	{
		//printf("%i " , i);
		if(fscanf(info, "%i", &(temp)) != 1)
		{
			char errorStr[2048];
			strcpy(errorStr, "Invalid number of data widths (line 2) in info file.");
			MessageBox( errorStr, NULL, MB_OK );
			return(1);
		}

		appInfo->showDBColumnWidths[i] = temp;
		if(temp > 160)	
		{
				strcpy(errorStr, "Invalid field width in Info file");
				MessageBox( errorStr, NULL, MB_OK );
				return(1);
		}

 		appInfo->showDBColumnWidths[i] = FlipWord(appInfo->showDBColumnWidths[i]);
 	}                                               
  	
 	// third line is the data entry screen field width
	if(	fscanf(info, "%i\n", &(temp)) != 1)
	{
		char errorStr[2048];
			strcpy(errorStr, "Invalid data screen width (line 3) in info file.");
			MessageBox( errorStr, NULL, MB_OK );
			return(1);
	}
	appInfo->showDataWidth = temp;
	//		printf("%i ", appInfo->showDataWidth);
	if((temp < 0) || (temp > 160))
	{
			strcpy(errorStr, "Invalid first show data width in Info file");
			MessageBox( errorStr, NULL, MB_OK );
			return(1);
	}
 	appInfo->showDataWidth = FlipWord(appInfo->showDataWidth);  
 	
 	// 4th line is which column to display as first movable column in the database screen
 	if(fscanf(info, "%i\n", &(temp)) != 1)
	{
		char errorStr[2048];
		strcpy(errorStr, "Invalid first display column (line 4) in info file.");
		MessageBox( errorStr, NULL, MB_OK );
		return(1);
	}
	appInfo->firstColumnToShow = temp;
	//appInfo->firstColumnToShow = 0xffff;

	if((temp < 0) || (temp > 19))
	{
		strcpy(errorStr, "Error: invalid 'first show column' in Info file.");
		MessageBox( errorStr, NULL, MB_OK );
		return(1);
	}

   appInfo->firstColumnToShow = FlipWord(appInfo->firstColumnToShow); 

  	// 5th line is appInfo flags variable (readonly, struct lock etc... bit field
 	if(fscanf(info, "%i\n", &(temp)) != 1)
	{
		char errorStr[2048];
		strcpy(errorStr, "Invalid AppInfo flag  (line 5) in info file.");
			MessageBox( errorStr, NULL, MB_OK );
			return(1);
	}
	appInfo->flags = temp;

 	appInfo->flags = FlipWord(appInfo->flags); 

	// 6th and 7th lines are 'extra data'
	// first line is the field types	
	for(i=0; i<appInfo->numFields; i++)
 	{
 		if(fscanf(info, "%i", &(temp)) != 1)
		{
  
			char errorStr[2048];
			strcpy(errorStr, "Invalid extra data 1 fields (line 5) in info file.");
			MessageBox( errorStr, NULL, MB_OK ); 
			return(1);


		}
		appInfo->fieldExtraData[i] = temp;
 		appInfo->fieldExtraData[i] = SwapDWordToIntel(appInfo->fieldExtraData[i]);
 	} 
	for(i=0; i<appInfo->numFields; i++)
 	{
		if(fscanf(info, "%i", &(temp)) != 1)
		{
			  
			char errorStr[2048];
			strcpy(errorStr, "Invalid extra data 2 fields (line 6) in info file.");
			MessageBox( errorStr, NULL, MB_OK ); 
			return(1);
		}
		appInfo->fieldExtraData2[i] = temp;
 		appInfo->fieldExtraData2[i] = SwapDWordToIntel(appInfo->fieldExtraData2[i]);
 	} 



	// skip the final space char
	//fgetc(info);
	//fgetc(info);
	fgets(binStr, 4000, info);


	fgets(binStr, 4000, info);
	binLen = strlen(binStr);
	BinaryDataSize = binLen/2;

	for(i=0; i<binLen; i+=2)
	{
			if(	(binStr[i] >= '0') && (binStr[i] <= '9'))
			{
				BinaryDataString[i/2] = 16*(binStr[i] - '0');
			}
			else
			{
				BinaryDataString[i/2]  = 16*(binStr[i] - 'A' + 10);
			}
						
			if(	(binStr[i+1] >= '0') && (binStr[i+1] <= '9'))
			{
				BinaryDataString[i/2]  += (binStr[i+1] - '0');
			}
			else
			{
				BinaryDataString[i/2]  += (binStr[i+1] - 'A' + 10);
			}
	}

 	tempPopupPtr = popupListString;   
 	
 	while(fgets(tempPopupPtr, 100, info) != NULL)
 	{
 		if(strlen(tempPopupPtr) != 0)
 		{          
			//fputs("\n", output);
			//fputs(tempPopupPtr, output);
			//fputs("\n", output);

			listLength += strlen(tempPopupPtr)+1;

			if( (tempPopupPtr[0] == 'p') && (tempPopupPtr[1] == 'o') && (tempPopupPtr[2] == 'p') && (tempPopupPtr[3] == 'u') && (tempPopupPtr[4] == 'p') )
			{
				numItems = 0;
			}
			else
			{
				numItems++;
				if(numItems > 99)
				{
					strcpy(errorStr, "Error: popup list exceeds 100 items.");
					MessageBox( errorStr, NULL, MB_OK );

					return(1);

				}
			}
		
 			if(tempPopupPtr[strlen(tempPopupPtr)-1] == 0x0A)
 			{
 				tempPopupPtr[strlen(tempPopupPtr)-1] = '\0';
 			}
 			
 			tempPopupPtr += (strlen(tempPopupPtr)+1);	
		}
 	}    
 	
 	tempPopupPtr[0] = '\0';  

	// to be safe add in the remaining nulls to fill in a blank list to make
	// it be equal to the length of a char ptr
 	tempPopupPtr[1] = '\0'; 
	tempPopupPtr[2] = '\0'; 
	tempPopupPtr[3] = '\0'; 

	return(1);

}

unsigned long SwapDWordToMotor(unsigned long dwSubj)
{
	DWORD  dwAnswer = 0; 
	BYTE   oneByte; 
	int    maxCars = sizeof(DWORD) - 1;    // 3
	int	i;

	for (i=0; i <= maxCars; i++)
	{
		oneByte  = (BYTE)((dwSubj >> (i * 8)) & 0xFF); 
		dwAnswer |= ((DWORD)oneByte) << ((maxCars - i) * 8);
	}
	return(dwAnswer);
}


unsigned long SwapDWordToIntel(DWORD dwSubj)
{
	DWORD  dwAnswer = 0; 
	BYTE   oneByte; 
	int    maxCars = sizeof(DWORD) - 1;    // 3
	int i;

	for (i=0; i <= maxCars; i++)
	{
		oneByte = (BYTE)((dwSubj >> ((maxCars - i) * 8)) & 0xFF); 
		dwAnswer |= oneByte << (i * 8);
	}
	return(dwAnswer);
}


WORD FlipWord(WORD wSubj)
{
	WORD wAnswer;

	wAnswer = ((wSubj >> 8) & 0xff) | ((wSubj & 0xff) << 8);

	return(wAnswer);
}

void Enquote(char* from, char* to)
{
	strcpy(to, "\"");
	strcat(to, from);
	strcat(to, "\"");
}






void MessageBox2(char* lpszText, char* lpszCaption , unsigned int nType )
{
	MessageBox(lpszText, lpszCaption, nType);
	exit(1);
}

void MessageBox(char* lpszText, char* lpszCaption , unsigned int nType )
{
	fprintf(stderr, "jconv: %s\n", lpszText);
	return;
}

void showsynopsis()
{
	fprintf(stderr,
"jconv [-e|-d] [-i <infofile>] [-t title] <infile> <outfile>

        -e                encode: csv -> pdb
        -d                decode: pdb -> csv
	-s                use semicolon as delimiter
	-i <infofile>     use <infofile> as infofile
	-t title          title is the db-name
        -v                print version information
	-h                print this message

");
	return;
}

char* basename(char* file)
{
	char* ptr = NULL;
	if((ptr = strrchr(file, '/')) != NULL) {
		return(ptr);
	} else {
		return(file);
	}
}

void showversion()
{
	fprintf(stderr, "
jconv: Converts JFile Pro database files to csv-files and vica versa
Version 1.0
copyright 1999 by Andreas Mock <AMock@osram.com>
with permission of J. J. Lehett <JLehett@land-j.com>
see also www.land-j.com

");
	return;
}
